#include "LZJumpBonusFix.h"

#include "skse/SafeWrite.h"
#include "skse/Utilities.h"
#include "skse/GameData.h"

namespace lznsJumpBonusFix {
	static const UInt32 QueryJumpHeight_Base = 0x6B5A39;
	static const UInt32 QueryJumpHeight_Return = QueryJumpHeight_Base + 0x5;
	static const UInt32 Original_Call = 0x004D5230;

	/** Modifies the (game setting) jump height by 1% of the JumpingBonus actor value. */
	static double ModifyJumpHeight(double currJumpHeight, Actor* actor) {
		if (!actor) {
			return currJumpHeight;
		}

		float bonus = 1.0f + (actor->actorValueOwner.GetCurrent(LookupActorValueByName("JumpingBonus")) / 100.0f);
		return bonus * currJumpHeight;
	}

	/**
	 * Assembly code for modifying the default jump height (fJumpHeightMin).
	 * The chain of instructions is as follows:
	 *
	 *     call 0x004059B0             <-- queries the fJumpHeightMin game setting
	 *     fstp qword ptr [esp+10]     <-- stores the resulting jump height
	 *     mov ecx,edi
	 *	   call 0x004D5230             <-- this code replaces this call with ModifyJumpHeight
	 *     fmul qword ptr [esp+10]     <-- after ModifyJumpHeight is called and the height modified, normal execution resumes here
	 *     mov ecx, esi
	 */
	static void __declspec(naked) JumpHeightASM() {
		__asm {
			// inject function
			push eax
			push ecx
			push edx
			push [esp + 0x18]
			push [esp + 0x24]
			push [esp + 0x24]
			call ModifyJumpHeight
			add esp,0xC
			pop edx
			pop ecx
			pop eax
			fstp qword ptr [esp + 0x10]
			
			// original code
			call Original_Call
			jmp	QueryJumpHeight_Return
		}
	}

	void Install() {
		WriteRelJump(QueryJumpHeight_Base, (UInt32) JumpHeightASM);
	}
}
